"""utilities are functions that need to be used by multiple files."""
from os import PathLike
from pathlib import Path
from typing import Union

from ansible_generator.log import setup_logger

logger = setup_logger(name=__name__)

StrPath = Union[str, PathLike[str]]


def join_cwd_and_directory_path(dir_path: StrPath) -> Path:
    """Join the current working directory with the provided path.

    Args:
        dir_path: The directory path.

    Returns:
        Path: The finalized path.
    """
    logger.debug("joining paths")
    joined_path = Path.cwd().joinpath(dir_path).resolve()
    return joined_path



"""Create a logger instance """
from logging import ERROR, INFO, Logger, basicConfig, getLogger
from os import getenv
from typing import Union

from sentry_sdk import init
from sentry_sdk.integrations.logging import LoggingIntegration

from ansible_generator.version import __version__


def configure_sentry() -> None:
    """Configure the Sentry telemetry, unless disabled."""
    disabled = getenv(key="DISABLE_ANSIBLE_GENERATE_TELEMETRY", default=False)
    if not disabled:
        sentry_logging = LoggingIntegration(level=INFO, event_level=ERROR)
        init(
            dsn="https://036bd28e074a4a4a99712dc05c9f768e@sentry.io/202195",
            integrations=[sentry_logging],
            release=__version__,
        )


def setup_logger(name: Union[str, None] = None, log_level: int = INFO) -> Logger:
    """Setup a new Logger instance.

    Args:
        name (optional): The name of the logger. Usually ``__file__`. Defaults to None.
        log_level (optional): THe logging level to use. Defaults to INFO.

    Returns:
        Logger: The configured logger.
    """
    log_format = "%(message)s"
    basicConfig(format=log_format)
    configure_sentry()
    logger = getLogger(name)
    logger.setLevel(log_level)
    return logger



from argparse import ArgumentParser
from logging import DEBUG, INFO

from ansible_generator.main import AnsibleGenerator
from ansible_generator.version import __version__


def cli() -> None:
    """Run the CLI and application function via ArgumentParser."""
    try:
        parser = ArgumentParser(
            prog="ansible-generate",
            description="Generate an ansible playbook directory structure",
        )

        parser.add_argument(
            "-a", "--alternate-layout", action="store_true", dest="alternate_layout"
        )
        parser.add_argument(
            "-i",
            "--inventories",
            nargs="+",
            default=["production", "staging"],
            dest="inventories",
            type=str,
        )
        parser.add_argument(
            "-r", "--roles", nargs="+", default=[], dest="roles", type=str
        )
        parser.add_argument("-v", "--verbose", action="store_true", dest="verbosity")
        parser.add_argument(
            "-p", "--projects", nargs="+", default=[], dest="projects", type=str
        )
        parser.add_argument(
            "--version",
            action="version",
            version=f"%(prog)s {__version__}",
        )

        args = parser.parse_args()

        verbosity = DEBUG if args.verbosity else INFO
        generator = AnsibleGenerator(
            inventories=args.inventories,
            alternate_layout=args.alternate_layout,
            projects=args.projects,
            roles=args.roles,
            verbosity=verbosity,
        )
        generator.run()
    except KeyboardInterrupt:
        print("Interrupt detected, exiting...")



# -*- coding: utf-8 -*-
"""main defines the entrypoint into the application."""
from logging import INFO, Logger
from typing import MutableSequence, Union

from ansible_generator.directories import create_directory_layout
from ansible_generator.files import create_file_layout
from ansible_generator.log import setup_logger


class AnsibleGenerator:
    """The AnsibleGenerator class is the primary orchestrator to create a
    directory structure.
    """

    projects: MutableSequence[str]
    inventories: MutableSequence[str]
    roles: MutableSequence[str]

    alternate_layout: bool
    verbosity: int
    logger: Logger

    def __init__(
        self,
        inventories: Union[MutableSequence[str], None] = None,
        projects: Union[MutableSequence[str], None] = None,
        roles: Union[MutableSequence[str], None] = None,
        alternate_layout: bool = False,
        verbosity: int = INFO,
    ) -> None:
        """Initialize an AnsibleGenerator instance

        Args:
            inventories (optional): The list of inventories to create, if desired.
                Defaults to None.
            projects (optional): The list of projects to create, if desired.
                Defaults to None.
            roles (optional): The list of roles to generate using ansible-galaxy, if
                desired. Defaults to None.
            alternate_layout (optional): Whether the alternative layout should be used.
                Defaults to False.
            verbosity (optional): The logging level to use. Defaults to INFO.
        """
        if projects is None:
            projects = []
        if inventories is None:
            inventories = ["production", "staging"]
        if roles is None:
            roles = []

        self.verbosity = verbosity
        self.logger = setup_logger(name=__name__, log_level=self.verbosity)
        self.logger.debug(
            (
                'msg="initializing generator" inventories="%s" '
                + 'alternate_layout="%s" projects="%s"'
            ),
            ", ".join(inventories),
            alternate_layout,
            ", ".join(projects),
        )
        self.projects = projects
        self.inventories = inventories
        self.alternate_layout = alternate_layout
        self.roles = roles

    def run(self) -> None:
        """Run the ansible-generator behavior."""
        self.logger.debug('msg="beginning create directory"')
        if create_directory_layout(
            projects=self.projects,
            inventories=self.inventories,
            alternate_layout=self.alternate_layout,
            verbosity=self.verbosity,
        ):
            create_file_layout(
                projects=self.projects,
                inventories=self.inventories,
                alternate_layout=self.alternate_layout,
                roles=self.roles,
                verbosity=self.verbosity,
            )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ansible_generator']

package_data = \
{'': ['*']}

install_requires = \
['ansible', 'sentry-sdk>=1.8.0,<2.0.0']

entry_points = \
{'console_scripts': ['ansible-generate = ansible_generator:cli']}

setup_kwargs = {
    'name': 'ansible-generator',
    'version': '3.0.3',
    'description': 'Ansible project generation tool',
    'long_description': "# Ansible Generator\n\n## Description\nAnsible Generator is a python program designed to simplify creating a new\nansible playbook by creating the necessary directory structure for the user\nbased on ansible's best practices, as outlined in [content organization best practices](https://docs.ansible.com/ansible/2.8/user_guide/playbooks_best_practices.html#content-organization).\n\n## Installation\n\n\n### PIP\n```\npip install -U ansible-generator\n```\n\n### Source\n\n#### Without Make\n```\ngit clone https://github.com/kkirsche/ansible-generator.git\ncd ansible-generator\npip install -U wheel\npython setup.py bdist_wheel --universal\npip install -U dist/*.whl\n```\n#### With Make\n```\ngit clone https://github.com/kkirsche/ansible-generator.git\ncd ansible-generator\nmake wheel\npip install -U dist/*.whl\n```\n\n## Usage\n\n### Help Text\n\n```\nusage: ansible-generate [-h] [-a] [-i INVENTORIES [INVENTORIES ...]]\n                        [-r ROLES [ROLES ...]] [-v]\n                        [-p PROJECTS [PROJECTS ...]] [--version]\n\nGenerate an ansible playbook directory structure\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -a, --alternate-layout\n  -i INVENTORIES [INVENTORIES ...], --inventories INVENTORIES [INVENTORIES ...]\n  -r ROLES [ROLES ...], --roles ROLES [ROLES ...]\n  -v, --verbose\n  -p PROJECTS [PROJECTS ...], --projects PROJECTS [PROJECTS ...]\n  --version             show program's version number and exit\n```\n\n#### Defaults\n\n* `alternate-layout` --- `False`\n* `verbose` --- `False`\n* `inventories` --- `['production', 'staging']`\n* `roles` --- `[]`\n* `projects` --- `[]`\n\n### Example\n\n#### Current directory\n\n```\nansible-generate\n```\n\n#### New-project\n\n```\nansible-generate -p playbook_name\n```\n\n#### Alternate Layout\n\n```\nansible-generate -a\n```\n\n#### Custom Inventories\n\n```\nansible-generate -i production staging lab\n```\n\n#### Roles\nThis portion of the tool relies on Ansible's `ansible-galaxy` command line\napplication\n```\nansible-generate -r role1 role2\n```\n\n#### Output\n\n```\n~/Downloads ❯❯❯ ansible-generate -i production staging lab -r common ubuntu centos -a -p network_security_baseline\ncreating directory /Users/example_user/Downloads/network_security_baseline/roles\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/production/group_vars\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/production/host_vars\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/staging/group_vars\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/staging/host_vars\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/lab/group_vars\ncreating directory /Users/example_user/Downloads/network_security_baseline/inventories/lab/host_vars\ncreating file /Users/example_user/Downloads/network_security_baseline/inventories/production/hosts\ncreating file /Users/example_user/Downloads/network_security_baseline/inventories/staging/hosts\ncreating file /Users/example_user/Downloads/network_security_baseline/inventories/lab/hosts\ncreating file /Users/example_user/Downloads/network_security_baseline/site.yml\nansible galaxy output for role common:\n- common was created successfully\nansible galaxy output for role ubuntu:\n- ubuntu was created successfully\nansible galaxy output for role centos:\n- centos was created successfully\n```\n",
    'author': 'Kevin Kirsche',
    'author_email': 'kev.kirsche@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/kkirsche/ansible-generator',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""directories is used to generate the necessary directory structures."""
from logging import INFO, Logger
from os import PathLike
from pathlib import Path
from typing import Iterable, MutableSequence, Set, Union

from ansible_generator.log import setup_logger
from ansible_generator.utilities import join_cwd_and_directory_path

StrPath = Union[str, PathLike[str]]


def create_directory_layout(
    projects: Iterable[str],
    inventories: MutableSequence[str],
    alternate_layout: bool = False,
    verbosity: int = INFO,
) -> bool:
    """Creates the directory layout.

    Args:
        inventories: Array of strings noting the names of the inventories.
        alternate_layout: Boolean noting whether this is the primary or
            alternate directory structure.

    Returns:
        A boolean to say that it succeeded or failed.
    """
    logger = setup_logger(name=__name__, log_level=verbosity)
    for itemNum, inventory in enumerate(inventories):
        inventories[itemNum] = inventory.split("/")[-1]
        if inventory == ".":
            inventories[itemNum] = "dot"
        if inventory == "..":
            inventories[itemNum] = "dotdot"
        if inventory == "*":
            inventories[itemNum] = "star"

    if alternate_layout:
        required_paths = get_alternate_inventories_directory_paths(
            logger=logger, inventories=inventories
        )
    else:
        required_paths = {"group_vars", "host_vars", "roles"}

    logger.debug(
        'msg="%s required directories" directories="%s"',
        len(required_paths),
        ", ".join(required_paths),
    )
    if projects:
        logger.debug('msg="projects was defined" projects="%s"', ", ".join(projects))

        final_paths: Set[str] = set()
        for project in projects:
            final_paths.update(
                f"{project}/{required_path}" for required_path in required_paths
            )
        required_paths = final_paths
        logger.debug(
            'msg="%s project required directories" directories="%s"',
            len(required_paths),
            ", ".join(required_paths),
        )

    create_paths = set(map(join_cwd_and_directory_path, required_paths))
    for cp in create_paths:
        success = create_directory(logger=logger, dir_path=cp)
        if not success:
            return False

    return True


def create_directory(logger: Logger, dir_path: StrPath) -> bool:
    """Recursively creates a directory path if does not exist.

    Args:
        dir_path: The path to the directory that we would like created.

    Returns:
        A boolean to say that it was successful (True) or it failed (False).
    """
    if not isinstance(dir_path, Path):
        dir_path = Path(dir_path)

    if dir_path.exists():
        logger.info("directory %s exists", dir_path)
        return True

    try:
        logger.info("creating directory %s", dir_path)
        dir_path.mkdir(parents=True, exist_ok=True)
        return True
    except PermissionError:
        logger.error(
            (
                "PermissionError: failed to create %s\n"
                + "Try using sudo to fix the issue 😃"
            ),
            dir_path,
        )
    except NotADirectoryError:
        logger.error(
            "usage error: non-directory target. Ansible Generate should be "
            + "directed to a directory"
        )
    except Exception:
        logger.error("failed to create %s", dir_path, exc_info=True)
    return False


def get_alternate_inventories_directory_paths(
    logger: Logger, inventories: Iterable[str]
) -> Set[str]:
    """

    Args:
         inventories: An array of string inventory names

    Returns:
        An array of directories that are required based on the provided
        inventory names.
    """
    logger.debug('msg="building alternate inventory layout directory paths"')
    inventory_paths = {"roles"}
    for inventory in inventories:
        inventory_paths.update(
            {
                f"inventories/{inventory}/group_vars",
                f"inventories/{inventory}/host_vars",
            }
        )
    return inventory_paths



"""files is used to generate the necessary file."""
from logging import INFO, Logger
from os import PathLike, utime
from pathlib import Path
from shlex import split
from shutil import which
from subprocess import Popen
from tempfile import TemporaryFile
from typing import Iterable, MutableSequence, Set, Tuple, Union

from ansible_generator.log import setup_logger
from ansible_generator.utilities import join_cwd_and_directory_path

StrOrBytesPath = Union[str, bytes, PathLike[str], PathLike[bytes]]


def create_file_layout(
    projects: Iterable[str],
    inventories: MutableSequence[str],
    roles: MutableSequence[str],
    alternate_layout: bool = False,
    verbosity: int = INFO,
) -> bool:
    """Create the file layout for the inputs.

    Args:
        projects: An iterable of project names.
        inventories: A mutable sequence of inventories.
        roles: A mutable sequence of roles.
        alternate_layout (optional): Use the alternate layout. Defaults to False.
        verbosity (optional): The logging level. Defaults to INFO.

    Returns:
        bool: True if the layout was created successfully, False otherwise.
    """
    logger = setup_logger(name=__name__, log_level=verbosity)
    minimum_paths = ["site.yml"]

    for itemNum, inventory in enumerate(inventories):
        inventories[itemNum] = inventory.split("/")[-1]
        if inventory == ".":
            inventories[itemNum] = "dot"
        if inventory == "..":
            inventories[itemNum] = "dotdot"
        if inventory == "*":
            inventories[itemNum] = "star"

    required_paths = set(minimum_paths)
    if alternate_layout:
        required_paths.update(
            get_alternate_inventories_file_paths(logger=logger, inventories=inventories)
        )
        required_paths.update(minimum_paths)
    else:
        required_paths.update(inventories)

    logger.debug(
        'msg="%s required files" files="%s"', len(required_paths), required_paths
    )

    if projects:
        logger.debug('msg="projects was defined" projects="%s"', projects)

        final_paths: Set[str] = set()
        for project in projects:
            final_paths.update(
                f"{project}/{required_path}" for required_path in required_paths
            )
        required_paths = final_paths
        logger.debug(
            'msg="%s project required files" files="%s"',
            len(required_paths),
            required_paths,
        )

    touch_paths = set(map(join_cwd_and_directory_path, required_paths))
    for tp in touch_paths:
        success = touch(logger=logger, filename=tp)
        if not success:
            return False

    for project in projects:
        for role in roles:
            success = create_role(
                rolename=role,
                directory=f"{project}/roles",
                logger=logger,
            )
            if not success:
                return False
    return True


def get_alternate_inventories_file_paths(
    logger: Logger, inventories: Iterable[Union[Path, str]]
) -> Set[str]:
    """Generate the required inventory paths for creation elsewhere.

    Args:
        logger: A logger.
        inventories: The iterable of inventory paths.

    Returns:
        Set[str]: The set of unique inventory paths to create.
    """
    logger.debug("building alternate inventory layout file paths")
    inventory_paths: Set[str] = {
        f"inventories/{inventory}/hosts" for inventory in inventories
    }
    return inventory_paths


def touch(
    logger: Logger,
    filename: Union[StrOrBytesPath, int],
    times: Union[Tuple[int, int], None] = None,
) -> bool:
    """Touch the file at the location provided.

    Args:
        logger: A logger.
        filename: The filename to touch.
        times (optional): The access and modification times or None. Defaults to None.

    Returns:
        bool: True if the file was touched, False if there was an error.
    """
    try:
        logger.info("creating file %s", filename)
        with open(filename, "a") as f:
            try:
                utime(filename, times)
            finally:
                f.close()
        return True
    except Exception:
        logger.error("failed to create file", exc_info=True)
        return False


def create_role(
    rolename: str, directory: Union[StrOrBytesPath, None], logger: Logger
) -> bool:
    """Create a role using ansible-galaxy.

    Args:
        rolename: The name of the role to generate.
        directory: The directory where the role should be created.
        logger: A logger.

    Returns:
        bool: True if the role was created successfully, False if there was an error.
    """
    with TemporaryFile() as stdoutf:
        with TemporaryFile() as stderrf:
            galaxy_executable = which("ansible-galaxy")
            if galaxy_executable is None:
                logger.critical(
                    (
                        "ansible-galaxy executable was not found in your path, "
                        "skipping role creation"
                    )
                )
                return False
            cmd = split(f"{galaxy_executable} init {rolename}")
            process = Popen(
                cmd,
                universal_newlines=True,
                shell=False,  # nosec
                cwd=directory,
                stdout=stdoutf,
                stderr=stderrf,
            )
            process.wait()

            stdoutf.flush()
            stdoutf.seek(0)
            stderrf.flush()
            stderrf.seek(0)

            stdout = stdoutf.read().decode("utf-8")
            stderr = stderrf.read().decode("utf-8")

            print(f"ansible-galaxy output for role {rolename}:")
            if stdout:
                print(stdout.strip())
            if stderr:
                print(stderr.strip())
                return False
    return True
